@@ -157,7 +157,7 @@ class Parser extends Lexer {
             neg = false;
         }
 
-        if (token.type == TokenType.CC_CLOSE) {
+        if (token.type == TokenType.CC_CLOSE && !syntax.op2OptionECMAScript()) {
             if (!codeExistCheck(']', true)) newSyntaxException(ERR_EMPTY_CHAR_CLASS);
             env.ccEscWarn("]");
             token.type = TokenType.CHAR; /* allow []...] */
@@ -429,6 +429,9 @@ class Parser extends Lexer {
                 break;
             case '!':  /*         preceding read */
                 node = new AnchorNode(AnchorType.PREC_READ_NOT);
+                if (syntax.op2OptionECMAScript()) {
+                    env.pushPrecReadNotNode(node);
+                }
                 break;
             case '>':  /* (?>...) stop backtrack */
                 node = new EncloseNode(EncloseType.STOP_BACKTRACK); // node_new_enclose
@@ -579,10 +582,16 @@ class Parser extends Lexer {
         if (node.getType() == NodeType.ANCHOR) {
             AnchorNode an = (AnchorNode) node;
             an.setTarget(target);
+            if (syntax.op2OptionECMAScript() && an.type == AnchorType.PREC_READ_NOT) {
+                env.popPrecReadNotNode(an);
+            }
         } else {
             EncloseNode en = (EncloseNode)node;
             en.setTarget(target);
             if (en.type == EncloseType.MEMORY) {
+                if (syntax.op2OptionECMAScript()) {
+                    en.containingAnchor = env.currentPrecReadNotNode();
+                }
                 /* Don't move this to previous of parse_subexp() */
                 env.setMemNode(en.regNum, node);
             }
@@ -750,13 +759,37 @@ class Parser extends Lexer {
             break;
 
         case BACKREF:
-            int[]backRefs = token.getBackrefNum() > 1 ? token.getBackrefRefs() : new int[]{token.getBackrefRef1()};
-            node = new BackRefNode(token.getBackrefNum(),
-                            backRefs,
-                            token.getBackrefByName(),
-                            token.getBackrefExistLevel(), // #ifdef USE_BACKREF_AT_LEVEL
-                            token.getBackrefLevel(),      // ...
-                            env);
+            if (syntax.op2OptionECMAScript() && token.getBackrefNum() == 1 && env.memNodes != null) {
+                EncloseNode encloseNode = (EncloseNode) env.memNodes[token.getBackrefRef1()];
+                boolean shouldIgnore = false;
+                if (encloseNode != null && encloseNode.containingAnchor != null) {
+                    shouldIgnore = true;
+                    for (Node anchorNode : env.precReadNotNodes) {
+                        if (anchorNode == encloseNode.containingAnchor) {
+                            shouldIgnore = false;
+                            break;
+                        }
+                    }
+                }
+                if (shouldIgnore) {
+                    node = StringNode.EMPTY;
+                } else {
+                    node = new BackRefNode(token.getBackrefNum(),
+                                    new int[]{token.getBackrefRef1()},
+                                    token.getBackrefByName(),
+                                    token.getBackrefExistLevel(), // #ifdef USE_BACKREF_AT_LEVEL
+                                    token.getBackrefLevel(),      // ...
+                                    env);
+                }
+            } else {
+                int[]backRefs = token.getBackrefNum() > 1 ? token.getBackrefRefs() : new int[]{token.getBackrefRef1()};
+                node = new BackRefNode(token.getBackrefNum(),
+                                backRefs,
+                                token.getBackrefByName(),
+                                token.getBackrefExistLevel(), // #ifdef USE_BACKREF_AT_LEVEL
+                                token.getBackrefLevel(),      // ...
+                                env);
+            }
 
             break;
 
@@ -857,6 +890,9 @@ class Parser extends Lexer {
         while (token.type == TokenType.OP_REPEAT || token.type == TokenType.INTERVAL) { // repeat:
             if (target.isInvalidQuantifier()) newSyntaxException(ERR_TARGET_OF_REPEAT_OPERATOR_INVALID);
 
+            if (!group && syntax.op2OptionECMAScript() && target.getType() == NodeType.QTFR) {
+                newSyntaxException(ERR_NESTED_REPEAT_NOT_ALLOWED);
+            }
             QuantifierNode qtfr = new QuantifierNode(token.getRepeatLower(),
                                                      token.getRepeatUpper(),
                                                      token.type == TokenType.INTERVAL);
@@ -871,7 +907,7 @@ class Parser extends Lexer {
                 qn = en;
             }
 
-            if (ret == 0) {
+            if (ret == 0 || (syntax.op2OptionECMAScript() && ret == 1)) {
                 target = qn;
             } else if (ret == 2) { /* split case: /abc+/ */
                 target = ConsAltNode.newListNode(target, null);
