@@ -335,8 +335,12 @@ public final class CClassNode extends Node {
 
         if (Config.NON_UNICODE_SDW) {
             switch(ctype) {
-            case CharacterType.D:
             case CharacterType.S:
+                if (!not && env.syntax.op2OptionECMAScript()) {
+                    // treat \u2028 and \u2029 as whitespace
+                    addCodeRange(env, 8232, 8233);
+                }
+            case CharacterType.D:
             case CharacterType.W:
                 ctype ^= CharacterType.SPECIAL_MASK;
                 if (not) {
